@@ -3,13 +3,12 @@
    Data Link Provider Interface (DLPI) network interface code. */
 
 /*
- * Copyright (c) 2009-2011 by Internet Systems Consortium, Inc. ("ISC")
- * Copyright (c) 2004,2007 by Internet Systems Consortium, Inc. ("ISC")
+ * Copyright (c) 2004-2017 by Internet Systems Consortium, Inc. ("ISC")
  * Copyright (c) 1996-2003 by Internet Software Consortium
  *
- * Permission to use, copy, modify, and distribute this software for any
- * purpose with or without fee is hereby granted, provided that the above
- * copyright notice and this permission notice appear in all copies.
+ * This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/.
  *
  * THE SOFTWARE IS PROVIDED "AS IS" AND ISC DISCLAIMS ALL WARRANTIES
  * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
@@ -125,54 +124,58 @@
 # endif
 
 #if defined(USE_DLPI_PFMOD) || defined(USE_DLPI_RAW)
-static int strioctl PROTO ((int fd, int cmd, int timeout, int len, char *dp));
+static int strioctl (int fd, int cmd, int timeout, int len, char *dp);
 #endif
 
 #define DLPI_MAXDLBUF		8192	/* Buffer size */
 #define DLPI_MAXDLADDR		1024	/* Max address size */
-#define DLPI_DEVDIR		"/dev/"	/* Device directory */
+
+/* Device directory */
+#if defined(USE_DEV_NET)
+#define DLPI_DEVDIR		"/dev/net/"  /* Solaris 11 + */
+#else
+#define DLPI_DEVDIR		"/dev/"      /* Pre Solaris 11 */
+#endif
 
 static int dlpiopen(const char *ifname);
-static int dlpiunit PROTO ((char *ifname));
-static int dlpiinforeq PROTO ((int fd));
-static int dlpiphysaddrreq PROTO ((int fd, unsigned long addrtype));
-static int dlpiattachreq PROTO ((int fd, unsigned long ppa));
-static int dlpibindreq PROTO ((int fd, unsigned long sap, unsigned long max_conind,
-			       unsigned long service_mode, unsigned long conn_mgmt,
-			       unsigned long xidtest));
+static int dlpiunit (char *ifname);
+static int dlpiinforeq (int fd);
+static int dlpiphysaddrreq (int fd, unsigned long addrtype);
+static int dlpiattachreq (int fd, unsigned long ppa);
+static int dlpibindreq (int fd, unsigned long sap, unsigned long max_conind,
+			unsigned long service_mode, unsigned long conn_mgmt,
+			unsigned long xidtest);
 #if defined(UNUSED_DLPI_INTERFACE)
 /* These functions are unused at present, but may be used at a later date.
  * defined out to avoid compiler warnings about unused static functions.
  */
-static int dlpidetachreq PROTO ((int fd));
-static int dlpiunbindreq PROTO ((int fd));
+static int dlpidetachreq (int fd);
+static int dlpiunbindreq (int fd);
 #endif
-static int dlpiokack PROTO ((int fd, char *bufp));
-static int dlpiinfoack PROTO ((int fd, char *bufp));
-static int dlpiphysaddrack PROTO ((int fd, char *bufp));
-static int dlpibindack PROTO ((int fd, char *bufp));
+static int dlpiokack (int fd, char *bufp);
+static int dlpiinfoack (int fd, char *bufp);
+static int dlpiphysaddrack (int fd, char *bufp);
+static int dlpibindack (int fd, char *bufp);
 #if defined(USE_DLPI_SEND) || defined(USE_DLPI_RECEIVE)
 /* These functions are not used if we're only sourcing the get_hw_addr()
  * function (for USE_SOCKETS).
  */
-static int dlpiunitdatareq PROTO ((int fd, unsigned char *addr,
-				   int addrlen, unsigned long minpri,
-				   unsigned long maxpri, unsigned char *data,
-				   int datalen));
-static int dlpiunitdataind PROTO ((int fd,
-				   unsigned char *dstaddr,
-				   unsigned long *dstaddrlen,
-				   unsigned char *srcaddr,
-				   unsigned long *srcaddrlen,
-				   unsigned long *grpaddr,
-				   unsigned char *data,
-				   int datalen));
+static int dlpiunitdatareq (int fd, unsigned char *addr, int addrlen, 
+			    unsigned long minpri, unsigned long maxpri, 
+			    unsigned char *data, int datalen);
+static int dlpiunitdataind (int fd,
+			    unsigned char *dstaddr,
+			    unsigned long *dstaddrlen,
+			    unsigned char *srcaddr,
+			    unsigned long *srcaddrlen,
+			    unsigned long *grpaddr,
+			    unsigned char *data,
+			    int datalen);
 #endif /* !USE_DLPI_HWADDR: USE_DLPI_SEND || USE_DLPI_RECEIVE */
-static int	expected PROTO ((unsigned long prim, union DL_primitives *dlp,
-				  int msgflags));
-static int	strgetmsg PROTO ((int fd, struct strbuf *ctlp,
-				  struct strbuf *datap, int *flagsp,
-				  char *caller));
+static int expected (unsigned long prim, union DL_primitives *dlp, 
+		     int msgflags);
+static int strgetmsg (int fd, struct strbuf *ctlp, struct strbuf *datap,
+		      int *flagsp, char *caller);
 
 /* Reinitializes the specified interface after an address change.   This
    is not required for packet-filter APIs. */
@@ -406,6 +409,10 @@ void if_deregister_send (info)
    XXX Changes to the filter program may require changes to the constant
    offsets used in if_register_send to patch the NIT program! XXX */
 
+#if defined(RELAY_PORT)
+#error "Relay port is not yet supported for DLPI"
+#endif
+
 void if_register_receive (info)
 	struct interface_info *info;
 {
@@ -528,6 +535,7 @@ ssize_t send_packet (interface, packet, raw, len, from, to, hto)
 {
 #ifdef USE_DLPI_RAW
 	double hh [32];
+	int fudge;
 #endif
 	double ih [1536 / sizeof (double)];
 	unsigned char *dbuf = (unsigned char *)ih;
@@ -535,7 +543,6 @@ ssize_t send_packet (interface, packet, raw, len, from, to, hto)
 	unsigned char dstaddr [DLPI_MAXDLADDR];
 	unsigned addrlen;
 	int result;
-	int fudge;
 
 	if (!strcmp (interface -> name, "fallback"))
 		return send_fallback (interface, packet, raw,
@@ -554,8 +561,6 @@ ssize_t send_packet (interface, packet, raw, len, from, to, hto)
 	fudge = dbuflen % 4; /* IP header must be word-aligned. */
 	memcpy (dbuf + fudge, (unsigned char *)hh, dbuflen);
 	dbuflen += fudge;
-#else
-	fudge = 0;
 #endif
 	assemble_udp_ip_header (interface, dbuf, &dbuflen, from.s_addr,
 				to -> sin_addr.s_addr, to -> sin_port,
@@ -695,7 +700,7 @@ ssize_t receive_packet (interface, buf, len, from, hfrom)
 	length -= offset;
 #endif
 	offset = decode_udp_ip_header (interface, dbuf, bufix,
-				       from, length, &paylen);
+				       from, length, &paylen, 1);
 
 	/*
 	 * If the IP or UDP checksum was bad, skip the packet...
@@ -798,9 +803,13 @@ dlpiopen(const char *ifname) {
 	ep = cp = ifname;
 	while (*ep)
 		ep++;
+
+/* Before Solaris 11 we strip off the digit to open the base dev name */
+#if !defined(USE_DEV_NET)
 	/* And back up to the first digit (unit number) */
 	while ((*(ep - 1) >= '0' && *(ep - 1) <= '9') || *(ep - 1) == ':')
 		ep--;
+#endif
 	
 	/* Copy everything up to the unit number */
 	while (cp < ep) {
