@@ -3,12 +3,12 @@
    BSD socket interface code... */
 
 /*
- * Copyright (c) 2004-2012 by Internet Systems Consortium, Inc. ("ISC")
+ * Copyright (c) 2004-2020 by Internet Systems Consortium, Inc. ("ISC")
  * Copyright (c) 1995-2003 by Internet Software Consortium
  *
- * Permission to use, copy, modify, and distribute this software for any
- * purpose with or without fee is hereby granted, provided that the above
- * copyright notice and this permission notice appear in all copies.
+ * This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/.
  *
  * THE SOFTWARE IS PROVIDED "AS IS" AND ISC DISCLAIMS ALL WARRANTIES
  * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
@@ -24,12 +24,6 @@
  *   <info@isc.org>
  *   https://www.isc.org/
  *
- * This software has been written for Internet Systems Consortium
- * by Ted Lemon in cooperation with Vixie Enterprises and Nominum, Inc.
- * To learn more about Internet Systems Consortium, see
- * ``https://www.isc.org/''.  To learn more about Vixie Enterprises,
- * see ``http://www.vix.com''.   To learn more about Nominum, Inc., see
- * ``http://www.nominum.com''.
  */
 
 /* SO_BINDTODEVICE support added by Elliot Poger (poger@leland.stanford.edu).
@@ -41,6 +35,7 @@
  */
 
 #include "dhcpd.h"
+#include <isc/util.h>
 #include <errno.h>
 #include <sys/ioctl.h>
 #include <sys/uio.h>
@@ -67,8 +62,13 @@
  * XXX: this is gross.  we need to go back and overhaul the API for socket
  * handling.
  */
+static int no_global_v6_socket = 0;
 static unsigned int global_v6_socket_references = 0;
 static int global_v6_socket = -1;
+#if defined(RELAY_PORT)
+static unsigned int relay_port_v6_socket_references = 0;
+static int relay_port_v6_socket = -1;
+#endif
 
 static void if_register_multicast(struct interface_info *info);
 #endif
@@ -127,7 +127,7 @@ void if_reinitialize_receive (info)
 /* Generic interface registration routine... */
 int
 if_register_socket(struct interface_info *info, int family,
-		   int *do_multicast)
+		   int *do_multicast, struct in6_addr *linklocal6)
 {
 	struct sockaddr_storage name;
 	int name_len;
@@ -161,10 +161,26 @@ if_register_socket(struct interface_info *info, int family,
 		addr6 = (struct sockaddr_in6 *)&name; 
 		addr6->sin6_family = AF_INET6;
 		addr6->sin6_port = local_port;
-		/* XXX: What will happen to multicasts if this is nonzero? */
-		memcpy(&addr6->sin6_addr,
-		       &local_address6, 
-		       sizeof(addr6->sin6_addr));
+#if defined(RELAY_PORT)
+		if (relay_port &&
+		    ((info->flags & INTERFACE_STREAMS) == INTERFACE_UPSTREAM))
+			addr6->sin6_port = relay_port;
+#endif
+		/* A server feature */
+		if (bind_local_address6) {
+			memcpy(&addr6->sin6_addr,
+			       &local_address6,
+			       sizeof(addr6->sin6_addr));
+		}
+		/* A client feature */
+		if (linklocal6) {
+			memcpy(&addr6->sin6_addr,
+			       linklocal6,
+			       sizeof(addr6->sin6_addr));
+		}
+		if (IN6_IS_ADDR_LINKLOCAL(&addr6->sin6_addr)) {
+			addr6->sin6_scope_id = if_nametoindex(info->name);
+		}
 #ifdef HAVE_SA_LEN
 		addr6->sin6_len = sizeof(*addr6);
 #endif
@@ -180,7 +196,7 @@ if_register_socket(struct interface_info *info, int family,
 	default:
 		addr = (struct sockaddr_in *)&name; 
 		addr->sin_family = AF_INET;
-		addr->sin_port = local_port;
+		addr->sin_port = relay_port ? relay_port : local_port;
 		memcpy(&addr->sin_addr,
 		       &local_address,
 		       sizeof(addr->sin_addr));
@@ -195,7 +211,7 @@ if_register_socket(struct interface_info *info, int family,
 	/* Make a socket... */
 	sock = socket(domain, SOCK_DGRAM, IPPROTO_UDP);
 	if (sock < 0) {
-		log_fatal("Can't create dhcp socket: %m");
+		log_fatal("Can't create dhcp socket for %s: %m", info->name);
 	}
 
 	/* Set the REUSEADDR option so that we don't fail to start if
@@ -203,7 +219,8 @@ if_register_socket(struct interface_info *info, int family,
 	flag = 1;
 	if (setsockopt(sock, SOL_SOCKET, SO_REUSEADDR,
 			(char *)&flag, sizeof(flag)) < 0) {
-		log_fatal("Can't set SO_REUSEADDR option on dhcp socket: %m");
+		log_fatal("Can't set SO_REUSEADDR on dhcp socket for"
+			  " %s: %m", info->name);
 	}
 
 	/* Set the BROADCAST option so that we can broadcast DHCP responses.
@@ -212,7 +229,8 @@ if_register_socket(struct interface_info *info, int family,
 	if (info->ifp &&
 	    (setsockopt(sock, SOL_SOCKET, SO_BROADCAST,
 			 (char *)&flag, sizeof(flag)) < 0)) {
-		log_fatal("Can't set SO_BROADCAST option on dhcp socket: %m");
+		log_fatal("Can't set SO_BROADCAST on dhcp socket for"
+			  " %s: %m", info->name);
 	}
 
 #if defined(DHCPv6) && defined(SO_REUSEPORT)
@@ -221,14 +239,17 @@ if_register_socket(struct interface_info *info, int family,
 	 * daemons can bind to their own sockets and get data for their
 	 * respective interfaces.  This does not (and should not) affect
 	 * DHCPv4 sockets; we can't yet support BSD sockets well, much
-	 * less multiple sockets.
+	 * less multiple sockets. Make sense only with multicast.
+	 * RedHat defines SO_REUSEPORT with a kernel which does not support
+	 * it and returns ENOPROTOOPT so in this case ignore the error.
 	 */
-	if (local_family == AF_INET6) {
+	if ((local_family == AF_INET6) && *do_multicast) {
 		flag = 1;
-		if (setsockopt(sock, SOL_SOCKET, SO_REUSEPORT,
-			       (char *)&flag, sizeof(flag)) < 0) {
-			log_fatal("Can't set SO_REUSEPORT option on dhcp "
-				  "socket: %m");
+		if ((setsockopt(sock, SOL_SOCKET, SO_REUSEPORT,
+			        (char *)&flag, sizeof(flag)) < 0) &&
+		    (errno != ENOPROTOOPT)) {
+			log_fatal("Can't set SO_REUSEPORT on dhcp socket for"
+				  " %s: %m", info->name);
 		}
 	}
 #endif
@@ -247,8 +268,9 @@ if_register_socket(struct interface_info *info, int family,
 	/* Bind this socket to this interface. */
 	if ((local_family != AF_INET6) && (info->ifp != NULL) &&
 	    setsockopt(sock, SOL_SOCKET, SO_BINDTODEVICE,
-			(char *)(info -> ifp), sizeof(*(info -> ifp))) < 0) {
-		log_fatal("setsockopt: SO_BINDTODEVICE: %m");
+		       (char *)(info -> ifp), sizeof(*(info -> ifp))) < 0) {
+		log_fatal("Can't set SO_BINDTODEVICE on dhcp socket for"
+			  " %s : %m", (char *)(info->ifp));
 	}
 #endif
 
@@ -263,7 +285,8 @@ if_register_socket(struct interface_info *info, int family,
         if (info->address_count &&
 	    setsockopt(sock, IPPROTO_IP, IP_BROADCAST_IF, &info->addresses[0],
 		       sizeof(info->addresses[0])) < 0)
-		log_fatal("Can't set IP_BROADCAST_IF on dhcp socket: %m");
+		log_fatal("Can't set IP_BROADCAST_IF on dhcp socket for"
+			  " %s: %m", info->name);
 #endif
 
 #if defined(IP_PKTINFO) && defined(IP_RECVPKTINFO)  && defined(USE_V4_PKTINFO)
@@ -275,7 +298,8 @@ if_register_socket(struct interface_info *info, int family,
 		int on = 1;
 		if (setsockopt(sock, IPPROTO_IP, IP_RECVPKTINFO, 
 		               &on, sizeof(on)) != 0) {
-			log_fatal("setsockopt: IPV_RECVPKTINFO: %m");
+			log_fatal("Can't set IP_RECVPTKINFO on dhcp socket for"
+				  " %s: %m", info->name);
 		}
 	}
 #endif
@@ -292,29 +316,38 @@ if_register_socket(struct interface_info *info, int family,
 		/* RFC3542 */
 		if (setsockopt(sock, IPPROTO_IPV6, IPV6_RECVPKTINFO, 
 		               &on, sizeof(on)) != 0) {
-			log_fatal("setsockopt: IPV6_RECVPKTINFO: %m");
+			log_fatal("setsockopt: IPV6_RECVPKTINFO for %s: %m",
+				  info->name);
 		}
 #else
 		/* RFC2292 */
 		if (setsockopt(sock, IPPROTO_IPV6, IPV6_PKTINFO, 
 		               &on, sizeof(on)) != 0) {
-			log_fatal("setsockopt: IPV6_PKTINFO: %m");
+			log_fatal("setsockopt: IPV6_PKTINFO for %s: %m",
+				  info->name);
 		}
 #endif
 	}
 
-	if ((family == AF_INET6) &&
-	    ((info->flags & INTERFACE_UPSTREAM) != 0)) {
-		int hop_limit = 32;
-		if (setsockopt(sock, IPPROTO_IPV6, IPV6_MULTICAST_HOPS,
-			       &hop_limit, sizeof(int)) < 0) {
-			log_fatal("setsockopt: IPV6_MULTICAST_HOPS: %m");
-		}
-	}
 #endif /* DHCPv6 */
 
 	return sock;
 }
+
+#ifdef DHCPv6
+void set_multicast_hop_limit(struct interface_info* info, int hop_limit) {
+	if (setsockopt(info->wfdesc, IPPROTO_IPV6, IPV6_MULTICAST_HOPS,
+		       &hop_limit, sizeof(int)) < 0) {
+		log_fatal("setsockopt: IPV6_MULTICAST_HOPS for %s: %m",
+			  info->name);
+	}
+
+	log_debug("Setting hop count limit to %d for interface %s",
+		  hop_limit, info->name);
+
+}
+#endif /* DHCPv6 */
+
 #endif /* USE_SOCKET_SEND || USE_SOCKET_RECEIVE || USE_SOCKET_FALLBACK */
 
 #if defined (USE_SOCKET_SEND) || defined (USE_SOCKET_FALLBACK)
@@ -322,7 +355,7 @@ void if_register_send (info)
 	struct interface_info *info;
 {
 #ifndef USE_SOCKET_RECEIVE
-	info->wfdesc = if_register_socket(info, AF_INET, 0);
+	info->wfdesc = if_register_socket(info, AF_INET, 0, NULL);
 	/* If this is a normal IPv4 address, get the hardware address. */
 	if (strcmp(info->name, "fallback") != 0)
 		get_hw_addr(info->name, &info->hw_address);
@@ -368,7 +401,7 @@ void if_register_receive (info)
 
 #if defined(IP_PKTINFO) && defined(IP_RECVPKTINFO) && defined(USE_V4_PKTINFO)
 	if (global_v4_socket_references == 0) {
-		global_v4_socket = if_register_socket(info, AF_INET, 0);
+		global_v4_socket = if_register_socket(info, AF_INET, 0, NULL);
 		if (global_v4_socket < 0) {
 			/*
 			 * if_register_socket() fatally logs if it fails to
@@ -384,7 +417,7 @@ void if_register_receive (info)
 #else
 	/* If we're using the socket API for sending and receiving,
 	   we don't need to register this interface twice. */
-	info->rfdesc = if_register_socket(info, AF_INET, 0);
+	info->rfdesc = if_register_socket(info, AF_INET, 0, NULL);
 #endif /* IP_PKTINFO... */
 	/* If this is a normal IPv4 address, get the hardware address. */
 	if (strcmp(info->name, "fallback") != 0)
@@ -404,7 +437,6 @@ void if_deregister_receive (info)
 #if defined(IP_PKTINFO) && defined(IP_RECVPKTINFO) && defined(USE_V4_PKTINFO)
 	/* Dereference the global v4 socket. */
 	if ((info->rfdesc == global_v4_socket) &&
-	    (info->wfdesc == global_v4_socket) &&
 	    (global_v4_socket_references > 0)) {
 		global_v4_socket_references--;
 		info->rfdesc = -1;
@@ -448,7 +480,8 @@ if_register_multicast(struct interface_info *info) {
 	mreq.ipv6mr_interface = if_nametoindex(info->name);
 	if (setsockopt(sock, IPPROTO_IPV6, IPV6_JOIN_GROUP, 
 		       &mreq, sizeof(mreq)) < 0) {
-		log_fatal("setsockopt: IPV6_JOIN_GROUP: %m");
+		log_fatal("setsockopt: IPV6_JOIN_GROUP for %s: %m",
+			  info->name);
 	}
 
 	/*
@@ -467,7 +500,8 @@ if_register_multicast(struct interface_info *info) {
 		mreq.ipv6mr_interface = if_nametoindex(info->name);
 		if (setsockopt(sock, IPPROTO_IPV6, IPV6_JOIN_GROUP, 
 			       &mreq, sizeof(mreq)) < 0) {
-			log_fatal("setsockopt: IPV6_JOIN_GROUP: %m");
+			log_fatal("setsockopt: IPV6_JOIN_GROUP for %s: %m",
+				  info->name);
 		}
 	}
 }
@@ -477,17 +511,38 @@ if_register6(struct interface_info *info, int do_multicast) {
 	/* Bounce do_multicast to a stack variable because we may change it. */
 	int req_multi = do_multicast;
 
+	if (no_global_v6_socket) {
+		log_fatal("Impossible condition at %s:%d", MDL);
+	}
+
+#if defined(RELAY_PORT)
+	if (!relay_port ||
+	    ((info->flags & INTERFACE_STREAMS) == INTERFACE_DOWNSTREAM)) {
+#endif
 	if (global_v6_socket_references == 0) {
 		global_v6_socket = if_register_socket(info, AF_INET6,
-						      &req_multi);
+						      &req_multi, NULL);
 		if (global_v6_socket < 0) {
 			/*
 			 * if_register_socket() fatally logs if it fails to
 			 * create a socket, this is just a sanity check.
 			 */
 			log_fatal("Impossible condition at %s:%d", MDL);
+		} else if (bind_local_address6) {
+			char addr6_str[INET6_ADDRSTRLEN];
+
+			if (inet_ntop(AF_INET6,
+				      &local_address6,
+				      addr6_str,
+				      sizeof(addr6_str)) == NULL) {
+				log_fatal("inet_ntop: unable to convert "
+					  "local-address6");
+			}
+			log_info("Bound to [%s]:%d",
+				 addr6_str,
+				 (int) ntohs(local_port));
 		} else {
-			log_info("Bound to *:%d", ntohs(local_port));
+			log_info("Bound to *:%d", (int) ntohs(local_port));
 		}
 	}
 		
@@ -495,6 +550,30 @@ if_register6(struct interface_info *info, int do_multicast) {
 	info->wfdesc = global_v6_socket;
 	global_v6_socket_references++;
 
+#if defined(RELAY_PORT)
+	} else {
+	/*
+	 * If relay port is defined, we need to register one
+	 * IPv6 UPD socket to handle upstream server or relay agent
+	 * with a non-547 UDP local port.
+	 */
+	if ((relay_port_v6_socket_references == 0) &&
+	    ((info->flags & INTERFACE_STREAMS) == INTERFACE_UPSTREAM)) {
+		relay_port_v6_socket = if_register_socket(info, AF_INET6,
+							  &req_multi, NULL);
+		if (relay_port_v6_socket < 0) {
+			log_fatal("Impossible condition at %s:%d", MDL);
+		} else {
+			log_info("Bound to relay port *:%d",
+				 (int) ntohs(relay_port));
+		}
+	}
+	info->rfdesc = relay_port_v6_socket;
+	info->wfdesc = relay_port_v6_socket;
+	relay_port_v6_socket_references++;
+	}
+#endif
+
 	if (req_multi)
 		if_register_multicast(info);
 
@@ -515,15 +594,86 @@ if_register6(struct interface_info *info, int do_multicast) {
 	}
 }
 
+/*
+ * Register an IPv6 socket bound to the link-local address of
+ * the argument interface (used by clients on a multiple interface box,
+ * vs. a server or a relay using the global IPv6 socket and running
+ * *only* in a single instance).
+ */
+void
+if_register_linklocal6(struct interface_info *info) {
+	int sock;
+	int count;
+	struct in6_addr *addr6 = NULL;
+	int req_multi = 0;
+
+	if (global_v6_socket >= 0) {
+		log_fatal("Impossible condition at %s:%d", MDL);
+	}
+		
+	no_global_v6_socket = 1;
+
+	/* get the (?) link-local address */
+	for (count = 0; count < info->v6address_count; count++) {
+		addr6 = &info->v6addresses[count];
+		if (IN6_IS_ADDR_LINKLOCAL(addr6))
+			break;
+	}
+
+	if (!addr6) {
+		log_fatal("no link-local IPv6 address for %s", info->name);
+	}
+
+	sock = if_register_socket(info, AF_INET6, &req_multi, addr6);
+
+	if (sock < 0) {
+		log_fatal("if_register_socket for %s fails", info->name);
+	}
+
+	info->rfdesc = sock;
+	info->wfdesc = sock;
+
+	get_hw_addr(info->name, &info->hw_address);
+
+	if (!quiet_interface_discovery) {
+		if (info->shared_network != NULL) {
+			log_info("Listening on Socket/%d/%s/%s",
+				 global_v6_socket, info->name, 
+				 info->shared_network->name);
+			log_info("Sending on   Socket/%d/%s/%s",
+				 global_v6_socket, info->name,
+				 info->shared_network->name);
+		} else {
+			log_info("Listening on Socket/%s", info->name);
+			log_info("Sending on   Socket/%s", info->name);
+		}
+	}
+}
+
 void 
 if_deregister6(struct interface_info *info) {
-	/* Dereference the global v6 socket. */
-	if ((info->rfdesc == global_v6_socket) &&
-	    (info->wfdesc == global_v6_socket) &&
-	    (global_v6_socket_references > 0)) {
+	/* client case */
+	if (no_global_v6_socket) {
+		close(info->rfdesc);
+		info->rfdesc = -1;
+		info->wfdesc = -1;
+	} else if ((info->rfdesc == global_v6_socket) &&
+		   (info->wfdesc == global_v6_socket) &&
+		   (global_v6_socket_references > 0)) {
+		/* Dereference the global v6 socket. */
 		global_v6_socket_references--;
 		info->rfdesc = -1;
 		info->wfdesc = -1;
+#if defined(RELAY_PORT)
+	} else if (relay_port &&
+		   (info->rfdesc == relay_port_v6_socket) &&
+		   (info->wfdesc == relay_port_v6_socket) &&
+		   (relay_port_v6_socket_references > 0)) {
+		/* Dereference the relay port v6 socket. */
+		relay_port_v6_socket_references--;
+		info->rfdesc = -1;
+		info->wfdesc = -1;
+#endif
 	} else {
 		log_fatal("Impossible condition at %s:%d", MDL);
 	}
@@ -540,11 +690,23 @@ if_deregister6(struct interface_info *info) {
 		}
 	}
 
-	if (global_v6_socket_references == 0) {
-		close(global_v6_socket);
-		global_v6_socket = -1;
+	if (!no_global_v6_socket) {
+		if (global_v6_socket_references == 0) {
+			close(global_v6_socket);
+			global_v6_socket = -1;
 
-		log_info("Unbound from *:%d", ntohs(local_port));
+			log_info("Unbound from *:%d",
+				 (int) ntohs(local_port));
+		}
+#if defined(RELAY_PORT)
+		if (relay_port && (relay_port_v6_socket_references == 0)) {
+			close(relay_port_v6_socket);
+			relay_port_v6_socket = -1;
+
+			log_info("Unbound from relay port *:%d",
+				 (int) ntohs(relay_port));
+		}
+#endif
 	}
 }
 #endif /* DHCPv6 */
@@ -573,7 +735,8 @@ ssize_t send_packet (interface, packet, raw, len, from, to, hto)
 			if (setsockopt(interface->wfdesc, IPPROTO_IP,
 				       IP_PKTINFO, (char *)&pktinfo,
 				       sizeof(pktinfo)) < 0) 
-				log_fatal("setsockopt: IP_PKTINFO: %m");
+				log_fatal("setsockopt: IP_PKTINFO for %s: %m",
+					  (char*)(interface->ifp));
 		}
 #endif
 		result = sendto (interface -> wfdesc, (char *)raw, len, 0,
@@ -692,9 +855,11 @@ ssize_t send_packet6(struct interface_info *interface,
 		     struct sockaddr_in6 *to) {
 	struct msghdr m;
 	struct iovec v;
+	struct sockaddr_in6 dst;
 	int result;
 	struct in6_pktinfo *pktinfo;
 	struct cmsghdr *cmsg;
+	unsigned int ifindex;
 
 	/*
 	 * If necessary allocate space for the control message header.
@@ -717,9 +882,20 @@ ssize_t send_packet6(struct interface_info *interface,
 
 	/*
 	 * Set the target address we're sending to.
+	 * Enforce the scope ID for bogus BSDs.
 	 */
-	m.msg_name = to;
-	m.msg_namelen = sizeof(*to);
+	memcpy(&dst, to, sizeof(dst));
+	m.msg_name = &dst;
+	m.msg_namelen = sizeof(dst);
+	ifindex = if_nametoindex(interface->name);
+
+// Per OpenBSD patch-common_socket_c,v 1.7 2018/03/06 08:37:39 sthen Exp
+// always set the scope id.  We'll leave the test for no global socket
+// in place for all others.
+#ifndef __OpenBSD__
+	if (no_global_v6_socket)
+#endif
+		dst.sin6_scope_id = ifindex;
 
 	/*
 	 * Set the data buffer we're sending. (Using this wacky 
@@ -742,13 +918,14 @@ ssize_t send_packet6(struct interface_info *interface,
 	m.msg_control = control_buf;
 	m.msg_controllen = control_buf_len;
 	cmsg = CMSG_FIRSTHDR(&m);
+	INSIST(cmsg != NULL);
 	cmsg->cmsg_level = IPPROTO_IPV6;
 	cmsg->cmsg_type = IPV6_PKTINFO;
 	cmsg->cmsg_len = CMSG_LEN(sizeof(*pktinfo));
 	pktinfo = (struct in6_pktinfo *)CMSG_DATA(cmsg);
 	memset(pktinfo, 0, sizeof(*pktinfo));
-	pktinfo->ipi6_ifindex = if_nametoindex(interface->name);
-	m.msg_controllen = cmsg->cmsg_len;
+	pktinfo->ipi6_addr = local_address6;
+	pktinfo->ipi6_ifindex = ifindex;
 
 	result = sendmsg(interface->wfdesc, &m, 0);
 	if (result < 0) {
@@ -1046,7 +1223,7 @@ void maybe_setup_fallback ()
 	isc_result_t status;
 	struct interface_info *fbi = (struct interface_info *)0;
 	if (setup_fallback (&fbi, MDL)) {
-		fbi -> wfdesc = if_register_socket (fbi, AF_INET, 0);
+		fbi -> wfdesc = if_register_socket (fbi, AF_INET, 0, NULL);
 		fbi -> rfdesc = fbi -> wfdesc;
 		log_info ("Sending on   Socket/%s%s%s",
 		      fbi -> name,
