@@ -3,13 +3,12 @@
    Subroutines for dealing with connections. */
 
 /*
- * Copyright (c) 2009-2010 by Internet Systems Consortium, Inc. ("ISC")
- * Copyright (c) 2004,2007 by Internet Systems Consortium, Inc. ("ISC")
+ * Copyright (c) 2004-2017 by Internet Systems Consortium, Inc. ("ISC")
  * Copyright (c) 1999-2003 by Internet Software Consortium
  *
- * Permission to use, copy, modify, and distribute this software for any
- * purpose with or without fee is hereby granted, provided that the above
- * copyright notice and this permission notice appear in all copies.
+ * This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/.
  *
  * THE SOFTWARE IS PROVIDED "AS IS" AND ISC DISCLAIMS ALL WARRANTIES
  * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
@@ -25,16 +24,10 @@
  *   <info@isc.org>
  *   https://www.isc.org/
  *
- * This software has been written for Internet Systems Consortium
- * by Ted Lemon in cooperation with Vixie Enterprises and Nominum, Inc.
- * To learn more about Internet Systems Consortium, see
- * ``https://www.isc.org/''.  To learn more about Vixie Enterprises,
- * see ``http://www.vix.com''.   To learn more about Nominum, Inc., see
- * ``http://www.nominum.com''.
  */
 
 #include "dhcpd.h"
-
+#include <isc/util.h>
 #include <omapip/omapip_p.h>
 #include <arpa/inet.h>
 #include <arpa/nameser.h>
@@ -99,7 +92,6 @@ isc_result_t omapi_connect (omapi_object_t *c,
 		addrs -> addresses [0].addrlen = sizeof foo;
 		memcpy (addrs -> addresses [0].address, &foo, sizeof foo);
 		addrs -> addresses [0].port = port;
-		hix = 1;
 	}
 	status = omapi_connect_list (c, addrs, (omapi_addr_t *)0);
 	omapi_addr_list_dereference (&addrs, MDL);
@@ -157,6 +149,7 @@ isc_result_t omapi_connect_list (omapi_object_t *c,
 		if (local_addr) {
 			/* Only do TCPv4 so far. */
 			if (local_addr -> addrtype != AF_INET) {
+				close(obj->socket);
 				omapi_connection_dereference (&obj, MDL);
 				return DHCP_R_INVALIDARG;
 			}
@@ -175,6 +168,7 @@ isc_result_t omapi_connect_list (omapi_object_t *c,
 				  sizeof local_sin) < 0) {
 				omapi_connection_object_t **objp = &obj;
 				omapi_object_t **o = (omapi_object_t **)objp;
+				close(obj->socket);
 				omapi_object_dereference(o, MDL);
 				if (errno == EADDRINUSE)
 					return ISC_R_ADDRINUSE;
@@ -231,6 +225,16 @@ isc_result_t omapi_connect_list (omapi_object_t *c,
 			goto out;
 		status = omapi_connection_connect_internal ((omapi_object_t *)
 							    obj);
+		/*
+		 * inprogress is the same as success but used
+		 * to indicate to the dispatch code that we should
+		 * mark the socket as requiring more attention.
+		 * Routines calling this function should handle
+		 * success properly.
+		 */
+		if (status == ISC_R_INPROGRESS) {
+			status = ISC_R_SUCCESS;
+		}
 #if defined (TRACING)
 	}
 	omapi_connection_register (obj, MDL);
@@ -345,6 +349,7 @@ static void trace_connect_input (trace_type_t *ttype,
 	s += sizeof remote.sin_addr;
 	memcpy (&local.sin_addr, s, sizeof local.sin_addr);
 	s += sizeof local.sin_addr;
+	POST(s);
 
 	connect_index = ntohl (connect_index);
 	listener_index = ntohl (listener_index);
@@ -386,22 +391,23 @@ static void trace_connect_input (trace_type_t *ttype,
 	/* Find the matching connect object, if there is one. */
 	omapi_array_foreach_begin (omapi_connections,
 				   omapi_connection_object_t, lp) {
-	    for (i = 0; (lp -> connect_list &&
-			 i < lp -> connect_list -> count); i++) {
+	    for (i = 0; (lp->connect_list &&
+			 i < lp->connect_list->count); i++) {
 		    if (!memcmp (&remote.sin_addr,
-				 &lp -> connect_list -> addresses [i].address,
+				 &lp->connect_list->addresses[i].address,
 				 sizeof remote.sin_addr) &&
 			(ntohs (remote.sin_port) ==
-			 lp -> connect_list -> addresses [i].port))
-			lp -> state = omapi_connection_connected;
-			lp -> remote_addr = remote;
-			lp -> remote_addr.sin_family = AF_INET;
-			omapi_addr_list_dereference (&lp -> connect_list, MDL);
-			lp -> index = connect_index;
-			status = omapi_signal_in ((omapi_object_t *)lp,
-						  "connect");
-			omapi_connection_dereference (&lp, MDL);
-			return;
+			 lp->connect_list->addresses[i].port)) {
+			    lp->state = omapi_connection_connected;
+			    lp->remote_addr = remote;
+			    lp->remote_addr.sin_family = AF_INET;
+			    omapi_addr_list_dereference(&lp->connect_list, MDL);
+			    lp->index = connect_index;
+			    status = omapi_signal_in((omapi_object_t *)lp,
+						     "connect");
+			    omapi_connection_dereference (&lp, MDL);
+			    return;
+		    }
 		}
 	} omapi_array_foreach_end (omapi_connections,
 				   omapi_connection_object_t, lp);
@@ -618,7 +624,7 @@ isc_result_t omapi_connection_connect (omapi_object_t *h)
 
 static isc_result_t omapi_connection_connect_internal (omapi_object_t *h)
 {
-	int error;
+	int error = 0;
 	omapi_connection_object_t *c;
 	socklen_t sl;
 	isc_result_t status;
