@@ -3,13 +3,12 @@
    Memory allocation for the DHCP server... */
 
 /*
- * Copyright (c) 2009,2012 by Internet Systems Consortium, Inc. ("ISC")
- * Copyright (c) 2004-2007 by Internet Systems Consortium, Inc. ("ISC")
+ * Copyright (c) 2004-2017 by Internet Systems Consortium, Inc. ("ISC")
  * Copyright (c) 1996-2003 by Internet Software Consortium
  *
- * Permission to use, copy, modify, and distribute this software for any
- * purpose with or without fee is hereby granted, provided that the above
- * copyright notice and this permission notice appear in all copies.
+ * This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/.
  *
  * THE SOFTWARE IS PROVIDED "AS IS" AND ISC DISCLAIMS ALL WARRANTIES
  * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
@@ -25,12 +24,6 @@
  *   <info@isc.org>
  *   https://www.isc.org/
  *
- * This software has been written for Internet Systems Consortium
- * by Ted Lemon in cooperation with Vixie Enterprises and Nominum, Inc.
- * To learn more about Internet Systems Consortium, see
- * ``https://www.isc.org/''.  To learn more about Vixie Enterprises,
- * see ``http://www.vix.com''.   To learn more about Nominum, Inc., see
- * ``http://www.nominum.com''.
  */
 
 #include "dhcpd.h"
@@ -85,6 +78,7 @@ void relinquish_lease_hunks ()
 		dfree(c, MDL);
 	}
 }
+
 #endif
 
 struct lease *new_leases (n, file, line)
@@ -95,11 +89,13 @@ struct lease *new_leases (n, file, line)
 	struct lease *rval;
 #if defined (DEBUG_MEMORY_LEAKAGE_ON_EXIT)
 	rval = dmalloc ((n + 1) * sizeof (struct lease), file, line);
-	memset (rval, 0, sizeof (struct lease));
-	rval -> starts = n;
-	rval -> next = lease_hunks;
-	lease_hunks = rval;
-	rval++;
+	if (rval != NULL) {
+		memset (rval, 0, sizeof (struct lease));
+		rval -> starts = n;
+		rval -> next = lease_hunks;
+		lease_hunks = rval;
+		rval++;
+	}
 #else
 	rval = dmalloc (n * sizeof (struct lease), file, line);
 #endif
